@@ -3,9 +3,7 @@ package Config::Properties;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
-
-our (%properties, $defaults);
+our $VERSION = '0.03';
 
 #   new() - Constructor
 #
@@ -14,15 +12,17 @@ our (%properties, $defaults);
 #   for this object.
 sub new {
 	my $proto = shift;
-	my $defaultProperties = shift;
+	my $defaultProperties = shift || undef;
+	my $perlMode = shift || 0;
 	
 	my $class = ref($proto) || $proto;
-	my $self = {};
+	my $self = { 
+		'PERL_MODE' => $perlMode,
+		'defaults' => $defaultProperties,
+		'properties' => {}
+	};
 	bless($self, $class);
 
-	if ($defaultProperties) {
-		$defaults = $defaultProperties;
-	}
 	return $self;
 }
 
@@ -32,14 +32,15 @@ sub setProperty {
 	my $key = shift or die "Config::Properties.setProperty( key, value )";
 	my $value = shift or die "Config::Properties.setProperty( key, value )";
 
-	my $oldValue = $properties{ $key };
-	$properties{ $key } = $value;
+	my $oldValue = $self->{properties}{ $key };
+	$self->{properties}{ $key } = $value;
 	return $oldValue;
 }
 
 #	getProperties() - Return a hashref of all of the properties
 sub getProperties {
-	return \%properties;
+	my $self =  shift;
+	return $self->{properties};
 }
 
 #	load() - Load the properties from a filehandle
@@ -47,24 +48,24 @@ sub load {
 	my $self = shift;
 	my $file = shift or die "Config::Properties.load( file )";
 	while (<$file>) {
-		$self->process_line($_, $file)
+		$self->process_line($_, $file);
 	}
 }
 
 #	process_line() - Recursive function used to parse a line from the
 #					properties file.
 sub process_line {
+	my $self =  shift;
 	#print "XXX" . join("::", @_) . "XXX\n";
-	my $self = shift;
-	my $line = shift or die "Config::Properties.process_line( line, file )";;
+	my $line = shift or die "Config::Properties.process_line( line, file )";
 	my $file = shift or die "Config::Properties.process_line( line, file )";
 	$line =~ s/\015?\012$//;
-	$line =~ s/\\\\(?!$)/\\/g;
-	return if $line =~ /^\s*(\#|\!)/;
-	
-	if ($line =~ /\\\s*$/ and not $line =~ s/\\\\$/\\/g) { 
-		#print "Found match...";
-		$line =~ s/\\\s*$//;
+	if ($line =~ /^\s*(\#|\!|$)/) {
+	 	return;
+	}
+	 
+	if ($line =~ /(\\+)$/ and length($1) % 2) {
+		$line =~ s/\\$//;
 		my $newline = <$file>;
 		$newline =~ s/^\s*//;
 		$self->process_line($line . $newline, $file);
@@ -76,7 +77,37 @@ sub process_line {
 	#print "1: $1 2: $2 3: $3 4: $4\n";
 	die "Config::Properties.process_line: invalid property line" if not $1;
 
-	$properties{ $1 } = ($4 || "");
+	#$properties{ $1 } = ($4 || "");
+	#the javadoc for Properties states that both the name and value
+	#can be escaped. The regex above will break though if ':','=', or
+	#whitespace are included.
+	$self->{properties}{ unescape($1) } = (unescape($4) || "");
+}
+
+#	unescape() - converts escaped characters to their real counterparts.
+sub unescape {
+	my $value = shift;
+
+	while ($value =~ m/\\(.)/g) {
+		my $result = $1;
+	
+		if ($result eq 't') {
+	 	    $result = "\t";
+	 	} elsif ($result eq 'n') {
+	 	    $result = "\n";
+	 	} elsif ($result eq 'r') {
+	 	    $result = "\r";
+	 	} elsif ($result eq 's') {
+	 	    $result = " ";
+	 	}
+	 	
+	 	my $start = (pos $value) - 2;
+	 	pos $value = $start;
+	 	$value =~ s/\\./$result/;
+	 	pos $value = ($start + 1);
+	}
+	 
+	return $value; 
 }
 
 #	reallySave() - Utility function that performs the actual saving of
@@ -85,8 +116,8 @@ sub reallySave {
 	#print "XXX" . join("::", @_) . "XXX\n"; 
 	my $self = shift;
 	my $file = shift or die "Config::Properties.reallySave( file )";
-	foreach (keys %properties) {
-		print $file $_ . "=" . $properties{$_} . "\n";
+	foreach (keys %{$self->{properties}}) { 
+		print $file $_ . "=" . $self->{properties}{$_} . "\n";
 	}
 }
 
@@ -115,16 +146,17 @@ sub getProperty {
 	my $self = shift;
 	my $key = shift or die "Config::Properties.getProperty( key )";
 	my $default = shift;
-	my $value = $properties{ $key };
+	my $value = $self->{properties}{ $key };
 	if ($default and not $value) {
 		return $default;
 	}
-	return $properties{ $key };
+	return $self->{properties}{ $key }; 
 }
 
 #	propertyName() - Returns an array of the keys of the Properties
 sub propertyNames {
-	return keys %properties;
+	my $self = shift;
+	return keys %{$self->{properties}};
 }
 
 #	list() - Same as store() except that it doesn't include a header.
@@ -136,6 +168,20 @@ sub list {
 	$self->reallySave( $file );
 }
 
+#	setPerlMode() - Sets the value (true/false) of the PERL_MODE parameter.
+sub setPerlMode {
+	my $self = shift;
+	my $mode = shift || undef;
+	return $self->{PERL_MODE} = $mode ? $mode : 
+		$self->{PERL_MODE} ? 0 : 1; 
+}
+
+#	perlMode() - Returns the current PERL_MODE setting (Default is false)
+sub perlMode {
+	my $self = shift;
+	return $self->{PERL_MODE};
+}
+
 1;
 __END__
 
@@ -174,6 +220,18 @@ after the backslash, the line break, and any whitespace at the beginning of the
 
 When a property file is saved it is in the format "key=value" for each line.
 
+If a true third parameter is passed to the constructor, the Config::Properties object
+be created in PERL_MODE. This can be set at any time by passing a true or false value
+into the setPerlMode() instance method. If in PERL_MODE, the behavior of the object
+may be expanded, enhanced and/or just plain different than the Java API spec.
+
+The following is a list of the current behavior changed under PERL_MODE:
+
+* Ummm... nothing yet.
+
+The current (true/false) value of PERL_MODE can be retrieved with the perlMode() instance
+variable.
+
 =head1 AUTHOR
 
 C<Config::Properties> was developed by Randy Jay Yarger.
